/* -*- Mode: C; c-file-style: "stroustrup" -*- */
#include "config.h"

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <readline.h>
#include <history.h>
#include <popt.h>
#include "kernel/identifier.h"
#include "Numexp_Kernel.h"
#include <libbonobo.h>
#include <string.h>
#include "numexp-client1.h"
#include <unistd.h>
#include <stdlib.h>


GCompletion	  *function_completion		= NULL;
char 		  *command_line_eval_string 	= NULL;
char 		  *command_line_simp_string 	= NULL;
gboolean	   debug_noeval			= FALSE;
gboolean	   debug_fatal_warnings		= FALSE;
char 		  *rcfile 			= NULL;
Numexp_Kernel      kernel;
NumexpClient      *client;


void     line_entered    (char         *line);
gboolean input_read_char (GIOChannel   *source,
			  GIOCondition  condition,
			  gpointer      data);
void     handle_exception(CORBA_Environment *ev);

static char *prompt;


gboolean eval_string(gpointer exp) {
    char* expression = (char*) exp;
    CORBA_Environment  ev, ev1;

    CORBA_exception_init(&ev);
    CORBA_exception_init(&ev1);

    Numexp_Element element, el1;
    CORBA_char *answer;
    
    if (strlen(expression) == 0) {
	printf("void\n");
    } else {
	element = Numexp_Kernel_parseString(kernel, expression, &ev);
	if (ev._major != CORBA_NO_EXCEPTION) {
	    handle_exception(&ev);
	    el1 = 0;
	} else if (!element) {
	    el1 = 0;
	} else {
	    el1 = Numexp_Kernel_elementEval(kernel, element, &ev);
	    Numexp_Kernel_elementUnref(kernel, element, &ev1);
	    if (ev._major != CORBA_NO_EXCEPTION) {
		handle_exception(&ev);
	    }
	}
	if (el1) {
	    answer = Numexp_Kernel_elementStr(kernel, el1, &ev);
	    if (ev._major != CORBA_NO_EXCEPTION) {
		Numexp_Kernel_elementUnref(kernel, el1, &ev1);
		handle_exception(&ev);
		printf("void\n");
	    } else {
		printf("%s\n",answer);
		CORBA_free(answer);
		Numexp_Kernel_elementUnref(kernel, el1, &ev1);
	    }
	} else {
	    printf("void\n");
	}
    }

    Numexp_Kernel_setClient(kernel, CORBA_OBJECT_NIL, &ev);
    bonobo_object_unref(BONOBO_OBJECT(client));
    Numexp_Kernel_unref(kernel, &ev);
    bonobo_main_quit();
    rl_callback_handler_remove();

    return FALSE;
}


gboolean simplify_string(gpointer exp) {
    char* expression = (char*) exp;
    CORBA_Environment  ev, ev1;

    CORBA_exception_init(&ev);
    CORBA_exception_init(&ev1);

    Numexp_Element element, el1;
    CORBA_char *answer;
    
    if (strlen(expression) == 0) {
	printf("void\n");
    } else {
	element = Numexp_Kernel_parseString(kernel, expression, &ev);
	if (ev._major != CORBA_NO_EXCEPTION) {
	    handle_exception(&ev);
	    el1 = 0;
	} else if (!element) {
	    el1 = 0;
	} else {
	    el1 = Numexp_Kernel_elementSimplify(kernel, element, &ev);
	    Numexp_Kernel_elementUnref(kernel, element, &ev1);
	    if (ev._major != CORBA_NO_EXCEPTION) {
		handle_exception(&ev);
		el1 = 0;
	    }
	}
	if (el1) {
	    answer = Numexp_Kernel_elementStr(kernel, el1, &ev);
	    if (ev._major != CORBA_NO_EXCEPTION) {
		Numexp_Kernel_elementUnref(kernel, el1, &ev1);
		handle_exception(&ev);
		printf("void\n");
	    } else {
		printf("%s\n",answer);
		CORBA_free(answer);
		Numexp_Kernel_elementUnref(kernel, el1, &ev1);
	    }
	} else {
	    printf("void\n");
	}
    }

    Numexp_Kernel_setClient(kernel, CORBA_OBJECT_NIL, &ev);
    bonobo_object_unref(BONOBO_OBJECT(client));
    Numexp_Kernel_unref(kernel, &ev);
    bonobo_main_quit();
    rl_callback_handler_remove();

    return FALSE;
}


void changePrompt(void)
{
    if (prompt) g_free(prompt);
    prompt = g_strdup_printf("%s> ", client->cwd);
    rl_callback_handler_install(prompt, line_entered);
}

static struct poptOption options[] = {
    { 
	"eval",
	'e',
	POPT_ARG_STRING,
	&command_line_eval_string,
	1,
	"Evaluate expression, print result and exit",
	"EXPRESSION"
    },
    { 
	"simplify",
	's',
	POPT_ARG_STRING,
	&command_line_simp_string,
	1,
	"Simplify expression, print result and exit",
	"EXPRESSION"
    },
    { 
	"rc",
	'r',
	POPT_ARG_STRING,
	&rcfile,
	1,
	"Startup file to load instead of $HOME/.numexprc",
	"FILE"
    },
    POPT_AUTOHELP
    {
	NULL,
	'\0',
	0,
	NULL,
	0,
	NULL,
	NULL
    }
};


GCompletion *completion = NULL;

char* complete(char const *text, int state)
{
    static GList *current_completions = NULL;
    static char *prefix = NULL;
    /* This 'showing_cwd' is used to show the root definitions, too! */
    static gboolean showing_cwd = TRUE;
    char *new;

    if (state == 0) {
	g_free(prefix);
	prefix = NULL;
	showing_cwd = TRUE;
	current_completions = nxp_client_completion_complete(text, &prefix);
    }

    /* TODO: should us free current_completions list? (at least cell's?) */
    if (current_completions) {
	if (showing_cwd) {
	    if (prefix) {
		new = g_strdup_printf("%s%s", prefix, (char*)current_completions->data);
	    } else {
		new = g_strdup((char*)current_completions->data);
	    }
	    current_completions = g_list_next(current_completions);
	} else {
	    new = g_strdup((char*)current_completions->data);
	    current_completions = g_list_next(current_completions);
	}
	return new; 
    } else {
	if (showing_cwd) {
	    if (prefix) { prefix = NULL; return NULL; }
	    showing_cwd = FALSE;
	    current_completions = g_completion_complete(g_hash_table_lookup(client->completions, 
									    NXP_IDENTIFIER_PATH_SEP),
							text, NULL);
	    if (current_completions) {
		new = g_strdup((char*)current_completions->data);
		current_completions = g_list_next(current_completions);	    
		return new;
	    } else {
		return NULL;
	    }
	} else {
	    return NULL;
	}
    }
}

void nxp_client_show_matches(char** matches, int num_matches, int max_length) {
    rl_display_match_list(matches,num_matches,max_length);
    printf("%s%s",prompt, rl_line_buffer);
}

void initialize_readline(void)
{
    completion = g_completion_new(NULL);
    rl_completer_word_break_characters = g_strdup("`\"\t\n\\'@$><=;|&{([]-+*^, ");
    /* rl_completion_query_items          = 50;*/
    rl_completion_append_character     = 0;
    rl_completion_entry_function       = complete;
    rl_completion_display_matches_hook = nxp_client_show_matches;
}


void testing(void);


void handle_exception(CORBA_Environment *ev)
{
    if (ev->_major == CORBA_SYSTEM_EXCEPTION) {
	char *msg = bonobo_exception_get_text(ev);
	g_error("System exception: %s", msg);
    } else if (ev->_major == CORBA_USER_EXCEPTION) {
	if (!strcmp(ev->_id, ex_Numexp_EvalError)) {
	    Numexp_EvalError *err;
	    g_assert(ev->_any._type == TC_Numexp_EvalError);
	    err = ev->_any._value;
	    printf("%s: %s\n", ev->_id, err->detail);
	} else if (!strcmp(ev->_id, ex_Numexp_ParseError)) {
	    Numexp_ParseError *err;
	    g_assert(ev->_any._type == TC_Numexp_ParseError);
	    err = ev->_any._value;
	    printf("%s: %s\n", ev->_id, err->detail);
	} else {
	    printf("%s (unknown exception)\n", ev->_id);
	}
    }
    CORBA_exception_free(ev);
}


static gboolean delayed_init(gpointer data)
{
    CORBA_Environment  ev;
    CORBA_exception_init(&ev);

    /* Create Numexp::Client interface */
    client = NUMEXP_CLIENT(numexp_client_new());
    client->kernel = kernel;
    client->cwd = g_strdup(NXP_IDENTIFIER_PATH_SEP);

    Numexp_Kernel_setClient(kernel, (Numexp_Client) bonobo_object_corba_objref
			    (BONOBO_OBJECT(client)), &ev);
    Numexp_Kernel_registerClientFunction(kernel, NXP_IDENTIFIER_PATH_SEP "test_client_func", &ev);

    if (ev._major != CORBA_NO_EXCEPTION) {
	g_message("Numexp::Kernel::registerClientFunction error: %s",
		  CORBA_exception_id(&ev));
	CORBA_exception_free(&ev);
    }

    Numexp_Kernel_registerNamespaceListener(kernel, NXP_IDENTIFIER_PATH_SEP, &ev);
    if (ev._major != CORBA_NO_EXCEPTION) {
	g_message("Numexp::Kernel::registerNamespaceListener error: %s",
		  CORBA_exception_id(&ev));
	CORBA_exception_free(&ev);
    }

    /* Get initial completion list */
    numexp_client_get_completions(client, NXP_IDENTIFIER_PATH_SEP);

    return FALSE;
}

static void
load_module(Numexp_Kernel kernel, const gchar *module)
{
    CORBA_Environment ev;

    /* g_message("Load module '%s'", module); */
    CORBA_exception_init(&ev);
    Numexp_Kernel_loadModule(kernel, module, &ev);
    if (ev._major != CORBA_NO_EXCEPTION)
        g_warning("Failed to load module '%s'", module);
    CORBA_exception_free(&ev);
}

static void
load_startup_modules(Numexp_Kernel kernel)
{
    GIOChannel *chan;
    gchar *numexp_modules_conf = g_build_filename(g_get_home_dir(), ".numexp-modules.conf", NULL);
    gchar *line = NULL;
    gsize terminator_pos;

    chan = g_io_channel_new_file(numexp_modules_conf, "r", NULL);
    if (chan == NULL)
        chan = g_io_channel_new_file(NUMEXP_MODULES_CONF, "r", NULL);
    if (chan == NULL) {
        g_warning("Unable to locate numexp-modules.conf file in either %s or %s",
                  numexp_modules_conf, NUMEXP_MODULES_CONF);
        g_free(numexp_modules_conf);
        return;
    }
    g_free(numexp_modules_conf);
    
    while (g_io_channel_read_line(chan, &line, NULL, &terminator_pos, NULL) == G_IO_STATUS_NORMAL)
    {
        if (line[0] == '#' || terminator_pos == 0) {
            g_free(line);
            continue;
        }
        line[terminator_pos] = 0; /* kill the \n */
        /* TODO: strip whitespace */
        load_module(kernel, line);
        g_free(line);
    }
    g_io_channel_unref(chan);
}


/* ================= MAIN =================== */
int main(int argc, char *argv[], char *envp[])
{
    int c;
    poptContext pctxt;
    GIOChannel *iochan;
    CORBA_Environment  ev;
    gchar *history_file;

    /* ------ popt stuff --------- */
    pctxt = poptGetContext("numexp-client", argc, (const char **)argv, options, 0);
    poptReadDefaultConfig(pctxt, 0);
    while ( (c = poptGetNextOpt(pctxt)) > 0 );
    if (c < -1) {
	fprintf(stderr, "%s: %s\n",
		poptBadOption(pctxt, POPT_BADOPTION_NOALIAS),
		poptStrerror(c));
	poptPrintUsage(pctxt, stderr, 0);
	return 1;
    }
    poptFreeContext(pctxt);

    /* ===== initialize NumExp ===== */
    bonobo_init(&argc, argv);
    CORBA_exception_init(&ev);
    kernel = (Numexp_Kernel) bonobo_get_object("OAFIID:Numexp_Kernel",
					       "IDL:Numexp/Kernel:1.0", &ev);
    if (kernel == CORBA_OBJECT_NIL) {
	char *err = bonobo_exception_get_text(&ev);
	fprintf(stderr, "ERROR: Failed to activate the numexp kernel!\n"
		"CORBA error was: %s\n", err);
	g_free(err);
	exit(2);
    }

    g_idle_add(delayed_init, NULL);

    if (command_line_eval_string) {
	g_idle_add(eval_string, command_line_eval_string);
	bonobo_main();
    } else if (command_line_simp_string) {
	g_idle_add(simplify_string, command_line_simp_string);
	bonobo_main();
    } else {
	printf("Numexp %s \n", Numexp_Kernel_version(kernel, &ev));
	printf("(c)1998-2006 The NumExp Team\n\n");

	/* Integrate readline into the bonobo main event loop */
	prompt = g_strdup(NXP_IDENTIFIER_PATH_SEP "> ");
	rl_callback_handler_install(prompt , line_entered);
	iochan = g_io_channel_unix_new(STDIN_FILENO);
	g_io_add_watch(iochan, G_IO_IN, (GIOFunc) input_read_char, NULL);
	
	// Initialize completion
	initialize_readline();

	/* Read history file */
	history_file = g_build_filename(g_get_home_dir(), ".numexp_history", NULL);
	
	if (g_file_test(history_file, G_FILE_TEST_EXISTS))
	    read_history(history_file);
	using_history();

        load_startup_modules(kernel);

	/* ---------- read-eval-print cycle ---------- */
	bonobo_main();

	write_history(history_file);
	g_free(history_file);

    }
    return 0;
}


gboolean input_read_char(GIOChannel *source, GIOCondition condition, gpointer data)
{
    rl_callback_read_char();
    return TRUE;
}
    


void line_entered(char *line)
{
    CORBA_Environment  ev, ev1;

    CORBA_exception_init(&ev);
    CORBA_exception_init(&ev1);

    Numexp_Element element, el1;
    CORBA_char *result;
    
    if (!line) {
	/* --- cleanup and exit --- */
	Numexp_Kernel_setClient(kernel, CORBA_OBJECT_NIL, &ev);
	bonobo_object_unref(BONOBO_OBJECT(client));
	Numexp_Kernel_unref(kernel, &ev);
	bonobo_main_quit();
	rl_callback_handler_remove();
	printf("\n");
	return;
    }
    if (strlen(line) == 0) {
	return;
    }
    add_history((char *)line);

    element = Numexp_Kernel_parseString(kernel, line, &ev);
    if (ev._major != CORBA_NO_EXCEPTION) {
	handle_exception(&ev);
	return;
    }

    if (!element) {
	return;
    }
    
    el1 = Numexp_Kernel_elementSimplify(kernel, element, &ev);
    Numexp_Kernel_elementUnref(kernel, element, &ev1);

    if (ev._major != CORBA_NO_EXCEPTION) {
	handle_exception(&ev);
	return;
    }
    g_return_if_fail(el1);

    Numexp_Kernel_defineVariable(kernel, "::ans", el1, &ev);

    result = Numexp_Kernel_elementStr(kernel, el1, &ev);
    if (ev._major != CORBA_NO_EXCEPTION) {
	Numexp_Kernel_elementUnref(kernel, el1, &ev1);
        handle_exception(&ev);
	return;
    }
    if (strcmp(result,"void")) printf("ans = %s\n", result);

    CORBA_free(result);
    Numexp_Kernel_elementUnref(kernel, el1, &ev1);
}


