/* -*- Mode: C; c-file-style: "stroustrup"; -*- */

#include <math.h>

#include "int.h"
#include "real.h"
#include "frac.h"
#include "tree.h"
#include "op_bindings.h"
#include "types_misc.h"
#include "errors.h"
#include "i18n.h"

#define FRAC_REAL(func, op)    static NxpElement* func(NxpOp op, NxpElement* a, NxpElement *b, \
			    NxpEvalContext *context, GError **error) { \
    mpfr_t val; \
    NxpElement* retval;\
\
    mpfr_init_set_q(val, NXP_ELEMENT_FRAC(a)->value, GMP_RNDN); \
    mpfr_##op(val, val, NXP_ELEMENT_REAL(b)->value, GMP_RNDN); \
    retval = nxp_element_real_new_from_mpfr(val); \
    return retval; \
}

FRAC_REAL(add_f_r, add);
FRAC_REAL(mul_f_r, mul);
FRAC_REAL(sub_f_r, sub);

static NxpElement *sub_r_f(NxpOp op, NxpElement* a, NxpElement *b,
			   NxpEvalContext *context, GError **error) {
    mpfr_t val;
    NxpElement *retval;

    mpfr_init_set_q(val, NXP_ELEMENT_FRAC(b)->value, GMP_RNDN);
    mpfr_sub(val, NXP_ELEMENT_REAL(a)->value, val, GMP_RNDN);
    retval = nxp_element_real_new_from_mpfr(val);
    return retval;
}

/*static NxpElement *eq2(NxpOp op, NxpElement* a, NxpElement* b,
		       NxpEvalContext *context, GError **error) {
    nxp_real_t d = NXP_ELEMENT_FRAC_VALUE(a) - NXP_ELEMENT_REAL(b)->value;

    return nxp_element_int_new(d < - 0.00000001 || d > 0.00000001 ? 0 : 1);
}*/

static NxpElement *div_f_r(NxpOp op, NxpElement* a, NxpElement *b,
			   NxpEvalContext *context, GError **error)
{
    mpfr_t val;
    NxpElement *retval;
    if (mpfr_cmp_ui(NXP_ELEMENT_REAL(b)->value, 0) != 0) {
	mpfr_init_set_q(val, NXP_ELEMENT_FRAC(a)->value, GMP_RNDN);
	mpfr_div(val, val, NXP_ELEMENT_REAL(b)->value, GMP_RNDN);
	retval = nxp_element_real_new_from_mpfr(val);
    }
    else {
	g_set_error(error, nxp_error_domain, NXP_ERROR_DIVIDE_BY_ZERO, _("divide by zero"));
	retval = NULL;
    }
    return retval;
}

static NxpElement *div_r_f(NxpOp op, NxpElement* a, NxpElement *b,
			   NxpEvalContext *context, GError **error)
{
    mpfr_t val;
    NxpElement *retval;
    if (mpq_cmp_ui(NXP_ELEMENT_FRAC(b)->value, 0, 1) != 0) {
	mpfr_init_set_q(val, NXP_ELEMENT_FRAC(b)->value, GMP_RNDN);
	mpfr_div(val, NXP_ELEMENT_REAL(a)->value, val, GMP_RNDN);
	retval = nxp_element_real_new_from_mpfr(val);
    }
    else {
	g_set_error(error, nxp_error_domain, NXP_ERROR_DIVIDE_BY_ZERO, _("divide by zero"));
	retval = NULL;
    }
    return retval;
}

#if 0
static NxpElement *simp_f_r_hat(NxpOp op, NxpElement* a, NxpElement *b,
				NxpEvalContext *context, GError **error) {
    g_return_val_if_fail(NXP_ELEMENT_IS_FRAC(a), NULL);
    g_return_val_if_fail(NXP_ELEMENT_IS_REAL(b), NULL);
    /* (a1/a2)^b --> a1^b / a2^b */
    return nxp_frac_simplify(
	NXP_ELEMENT_FRAC(
	    nxp_element_frac_new(pow((double)NXP_ELEMENT_FRAC_NUM(a), 
				     (double)NXP_ELEMENT_REAL(b)->value),
				 pow((double)NXP_ELEMENT_FRAC_DENUM(a),
				     (double)NXP_ELEMENT_REAL(b)->value))));
}
#endif

#define BIND_F_I(op,simp) nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_FRAC, NXP_ELEMENT_TYPE_REAL, \
   			                              simp, simp,  0, NULL);
#define BIND_I_F(op,simp) nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_REAL, NXP_ELEMENT_TYPE_FRAC, \
   			                              simp, simp,  0, NULL);

#define BIND_SYM(op,simp) nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_FRAC, \
							     NXP_ELEMENT_TYPE_REAL, \
   			                              simp, simp,  NXP_OP_BIND_SYMMETRIC, NULL);



void nxp_element_frac_real_operator_init(void)
{
    BIND_SYM(NXP_OP_PLUS, add_f_r);
    BIND_SYM(NXP_OP_STAR, mul_f_r);

    BIND_F_I(NXP_OP_MINUS, sub_f_r);
    BIND_I_F(NXP_OP_MINUS, sub_r_f);

    BIND_F_I(NXP_OP_SLASH, div_f_r);
    BIND_I_F(NXP_OP_SLASH, div_r_f);

//    BIND_F_I(NXP_OP_HAT, simp_f_r_hat);
//    BIND_SYM(NXP_OP_EQ2, eq2);
/*   BIND_I_R(hat_int_real, NXP_OP_HAT); */

/*   BIND_R_I(gt_real_int, NXP_OP_GT); */
/*   BIND_I_R(gt_int_real, NXP_OP_GT); */

/*   BIND_R_I(lt_real_int, NXP_OP_LT); */
/*   BIND_I_R(lt_int_real, NXP_OP_LT); */

/*   BIND_R_I(gte_real_int, NXP_OP_GT_EQ); */
/*   BIND_I_R(gte_int_real, NXP_OP_GT_EQ); */

/*   BIND_R_I(lte_real_int, NXP_OP_LT_EQ); */
/*   BIND_I_R(lte_int_real, NXP_OP_LT_EQ); */

/*   BIND_SYM(nequals,NXP_OP_EXCL_EQ); */

/*   BIND_R_I(and_real_int, NXP_OP_SLASH_BSLASH); */
/*   BIND_I_R(and_int_real, NXP_OP_SLASH_BSLASH); */

/*   BIND_R_I(or_real_int, NXP_OP_BSLASH_SLASH); */
/*   BIND_I_R(or_int_real, NXP_OP_BSLASH_SLASH); */

}
