/* -*- Mode: C; c-file-style: "bsd"; -*- */

/* Matrix Int relation */

#include <math.h>

#include "matrix.h"
#include "vector.h"
#include "tree.h"

#include "ourlibc.h"

#include "op_bindings.h"
#include "types_misc.h"

#define BIND_SYM(func,op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_MATRIX, \
                                                        NXP_ELEMENT_TYPE_VECTOR,     \
							func, func,             \
							NXP_OP_BIND_SYMMETRIC,  \
							NULL);

#define BIND_M_V(f1,f2,op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_MATRIX, \
                                                        NXP_ELEMENT_TYPE_VECTOR,      \
							f1, f2, 0, NULL);

#define BIND_V_M(f1,f2,op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_VECTOR, \
                                                        NXP_ELEMENT_TYPE_MATRIX,      \
							f1, f2, 0, NULL);

#define BIND_SYM_FULL(f1,f2,op)    nxp_bind_operator_function(op,                     \
							      NXP_ELEMENT_TYPE_MATRIX,\
							      NXP_ELEMENT_TYPE_VECTOR,\
							      f1, f2,                 \
							      NXP_OP_BIND_SYMMETRIC,  \
							      NULL);



static NxpElement* vec_times_mat(NxpOp op, NxpElement* a, NxpElement* b,
				 NxpEvalContext *context, GError **error)
{
    return vector_times_matrix(NXP_ELEMENT_VECTOR(a), NXP_ELEMENT_MATRIX(b), context, error);
}

static NxpElement* mat_times_vec(NxpOp op, NxpElement* a, NxpElement* b,
				 NxpEvalContext *context, GError **error)
{
    return matrix_times_vector(NXP_ELEMENT_MATRIX(a), NXP_ELEMENT_VECTOR(b), context, error);
}

static NxpElement* or(NxpOp op, NxpElement *a, NxpElement *b, 
		       NxpEvalContext *context, GError **error) {
    return nxp_element_int_new((NXP_ELEMENT_MATRIX(a)->width * NXP_ELEMENT_MATRIX(a)->height)?
				   1:NXP_ELEMENT_VECTOR(b)->size);
}

static NxpElement* and(NxpOp op, NxpElement *a, NxpElement *b, 
		       NxpEvalContext *context, GError **error) {
	return nxp_element_int_new((NXP_ELEMENT_MATRIX(a)->width * NXP_ELEMENT_MATRIX(a)->height)?
				   NXP_ELEMENT_VECTOR(b)->size:0);
}

void nxp_element_matrix_vector_operator_init(void)
{
    // Somamos a cada elemento?
    // BIND_SYM( sum , NXP_OP_PLUS);
    // BIND_SYM( sum , NXP_OP_MINUS);

    // NXP_OP_MINUS_LEFT --> handled on other files

    BIND_V_M(vec_times_mat, NULL, NXP_OP_STAR);
    BIND_M_V(mat_times_vec, NULL, NXP_OP_STAR);

    //    NXP_OP_SLASH
    //    NXP_OP_HAT
    //    NXP_OP_DOT3
    //    NXP_OP_PLUS2_LEFT
    //    NXP_OP_MINUS2_LEFT
    //    NXP_OP_PLUS2_RIGHT
    //    NXP_OP_MINUS2_RIGHT
    //    NXP_OP_GT
    //    NXP_OP_GT_EQ
    //    NXP_OP_LT
    //    NXP_OP_LT_EQ
    //    NXP_OP_EQ2
    //    NXP_OP_EXCL_EQ
    BIND_SYM(and, NXP_OP_SLASH_BSLASH);
    BIND_SYM(or, NXP_OP_BSLASH_SLASH);
    //    NXP_OP_TILDE_LEFT
    //    NXP_OP_EXCL_RIGHT
    //    NXP_OP_EQ
    //    NXP_OP_COLON_EQ
}



