#ifndef __GSL__H__
#define __GSL__H__

#include "element.h"
#include <gsl/gsl_complex.h>
#include <gsl/gsl_permutation.h>
#include <gsl/gsl_histogram.h>
#include <gsl/gsl_histogram2d.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include "sf.h"

/**************************************
 *
 * Permutation type and related stuff
 *
 **/
#define NXP_ELEMENT_TYPE_PERMUTATION	(nxp_element_permutation_get_type())
#define NXP_ELEMENT_TYPE_HISTOGRAM	(nxp_element_histogram_get_type())
#define NXP_ELEMENT_TYPE_HISTOGRAM2D	(nxp_element_histogram2d_get_type())

typedef struct _NxpElementPermutation NxpElementPermutation;

#define NXP_ELEMENT_PERMUTATION(x)    (NXP_ELEMENT_TYPECAST(x, NXP_ELEMENT_TYPE_PERMUTATION, \
                                       NxpElementPermutation))
#define NXP_ELEMENT_IS_PERMUTATION(x) nxp_element_check_type((x), NXP_ELEMENT_TYPE_PERMUTATION)
struct _NxpElementPermutation {
    NxpElement element;
    gsl_permutation* data;
};

/************************************
 *
 * Histogram type and related stuff
 *
 **/
int NXP_ELEMENT_TYPE_HISTOGRAM;
int NXP_ELEMENT_TYPE_HISTOGRAM2D;

typedef struct _NxpElementHistogram   NxpElementHistogram;
typedef struct _NxpElementHistogram2d NxpElementHistogram2d;

#define NXP_ELEMENT_IS_HISTOGRAM(x) (NXP_ELEMENT_GET_TYPE(x) == NXP_ELEMENT_TYPE_HISTOGRAM)
#define NXP_ELEMENT_HISTOGRAM(x)    (NXP_ELEMENT_TYPECAST(x, NXP_ELEMENT_TYPE_HISTOGRAM, \
                                     NxpElementHistogram))
#define NXP_ELEMENT_IS_HISTOGRAM2D(x) (NXP_ELEMENT_GET_TYPE(x) == NXP_ELEMENT_TYPE_HISTOGRAM2D)
#define NXP_ELEMENT_HISTOGRAM2D(x)    (NXP_ELEMENT_TYPECAST(x, NXP_ELEMENT_TYPE_HISTOGRAM2D, \
                                       NxpElementHistogram2d))

struct _NxpElementHistogram {
    NxpElement element;
    gsl_histogram *histogram;
    gsl_histogram_pdf *pdf;
};

struct _NxpElementHistogram2d {
    NxpElement element;
    gsl_histogram2d *histogram;
};


extern int gsl_is_on_error;
extern char* gsl_error_message;

#define NXP_GSL_MODE  GSL_PREC_DOUBLE
//#define ETC NxpEvalContext *context, GError **error

#define GSL_RETURN(x)						\
    if (gsl_is_on_error) {					\
	g_set_error(error, nxp_error_domain, NXP_ERROR_ERROR,	\
		    "GSL error: %s", gsl_error_message);	\
	gsl_is_on_error = 0;					\
	return NULL;						\
    } else {							\
	return x;						\
    }

int nxp_element_permutation_get_type(void);
void nxp_gsl_functions_init(NxpEvalContext *context);

gsl_complex nxp_gsl_complex_from_real(double real);
gsl_complex nxp_gsl_complex_from_nxp_complex(NxpElementComplex* c);
NxpElement* nxp_complex_from_gsl_complex(gsl_complex c);

double* nxp_vector_to_double_buffer(NxpElementVector* v, NxpEvalContext *context, GError **error);
NxpElement* nxp_double_buffer_to_vector(double* buffer, int size);


gsl_vector* nxp_vector_to_gsl_vector(NxpElementVector *v, NxpEvalContext *context, GError **error);
NxpElement* gsl_vector_to_nxp_vector(gsl_vector* v);

gsl_matrix* nxp_matrix_to_gsl_matrix(NxpElementMatrix *v, GError **error);
NxpElement* gsl_matrix_to_nxp_matrix(gsl_matrix* v);

gsl_matrix_complex* nxp_matrix_to_gsl_cmatrix(NxpElementMatrix *v, GError **error);
NxpElement* gsl_cmatrix_to_nxp_matrix(gsl_matrix_complex* v);

gsl_vector_complex* nxp_vector_to_gsl_cvector(NxpElementVector *v, GError **error);
NxpElement* gsl_cvector_to_nxp_vector(gsl_vector_complex* v);

gint nxp_matrix_is_constant(NxpElementMatrix *m);


NxpElement* nxp_gsl_permutation_new(gsl_permutation* permut);
NxpElement* nxp_gsl_histogram_new(gsl_histogram* histogram);

NxpElement *nxp_pair_new(NxpElement *x, NxpElement *y);


#endif /* __GSL__H__ */





