/* -*- c -*- */
#include <gsl/gsl_poly.h>
#include "nxp_kernel.h"
#include "gsl.h"
#include "types/vector.h"


/*
 * NOTE: (TODO) --- When writing documentation, don't forget that the
 * first element from the vector is the x^0 coeficient, and the last one,
 * the x^(vec_size-1) one! 
 */
static NxpElement *
nxp_gsl_poly_roots_solve_generic(NxpElementVector * vec, ETC)
{
    gsl_poly_complex_workspace *ws;
    NxpElement     *res = NULL;
    double         *Z;
    int             i;

    double         *coeficientes = nxp_vector_to_double_buffer(vec, context, NULL); /* TODO: User GError */

    if (coeficientes) {

	/*
	 * TODO: Check if last element of vector is not zero 
	 */
	ws = gsl_poly_complex_workspace_alloc(NXP_ELEMENT_VECTOR_SIZE(vec));
	Z = (double *) g_new(double, 2 * (NXP_ELEMENT_VECTOR_SIZE(vec) - 1));
	res = nxp_element_vector_new(NXP_ELEMENT_VECTOR_SIZE(vec) - 1, NULL);

	gsl_poly_complex_solve(coeficientes,
			       NXP_ELEMENT_VECTOR_SIZE(vec), ws, Z);

	gsl_poly_complex_workspace_free(ws);

	for (i = 0; i < NXP_ELEMENT_VECTOR_SIZE(vec) - 1; ++i) {
	    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(res), i) =
		nxp_element_complex_new_from_dbl(Z[2 * i] + I * Z[2 * i + 1]);
	}
    }
    GSL_RETURN(res);
}

/*
 * x^3 + a x^2 + b x + c = 0 
 */
static NxpElement *
nxp_gsl_poly_roots_complex_solve_cubic(NxpElementReal * A,
				       NxpElementReal * B,
				       NxpElementReal * C, ETC)
{
    double          a = nxp_element_r_get_value_as_double(A);
    double          b = nxp_element_r_get_value_as_double(B);
    double          c = nxp_element_r_get_value_as_double(C);
    gsl_complex     r1,
                    r2,
                    r3;
    NxpElement     *vec = nxp_element_vector_new(3, NULL);

    if (vec) {
	gsl_poly_complex_solve_cubic(a, b, c, &r1, &r2, &r3);

	NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vec), 0) =
	    nxp_complex_from_gsl_complex(r1);
	NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vec), 1) =
	    nxp_complex_from_gsl_complex(r2);
	NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vec), 2) =
	    nxp_complex_from_gsl_complex(r3);
    }
    GSL_RETURN(vec);
}


/*
 * x^3 + a x^2 + b x + c = 0 
 */
static NxpElement *
nxp_gsl_poly_roots_solve_cubic(NxpElementReal * A, NxpElementReal * B,
			       NxpElementReal * C, ETC)
{
    double          a = nxp_element_r_get_value_as_double(A);
    double          b = nxp_element_r_get_value_as_double(B);
    double          c = nxp_element_r_get_value_as_double(C);
    double          res[3];
    NxpElement     *vector = NULL;

    if (gsl_poly_solve_cubic(a, b, c, &res[0], &res[1], &res[2]) == 1) {
	vector = nxp_element_vector_new(1, NULL);
	NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vector), 0) =
	    nxp_element_real_new(res[0]);
    } else {
	vector = nxp_element_vector_new(3, NULL);
	NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vector), 0) =
	    nxp_element_real_new(res[0]);
	NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vector), 1) =
	    nxp_element_real_new(res[1]);
	NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vector), 2) =
	    nxp_element_real_new(res[2]);
    }

    GSL_RETURN(vector);
}

static NxpElement *
nxp_gsl_poly_roots_complex_solve_quadratic(NxpElementReal * A,
					   NxpElementReal * B,
					   NxpElementReal * C, ETC)
{
    double          a = nxp_element_r_get_value_as_double(A);
    double          b = nxp_element_r_get_value_as_double(B);
    double          c = nxp_element_r_get_value_as_double(C);
    gsl_complex     r1,
                    r2;
    NxpElement     *vec = nxp_element_vector_new(2, NULL);

    gsl_poly_complex_solve_quadratic(a, b, c, &r1, &r2);

    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vec), 0) =
	nxp_complex_from_gsl_complex(r1);
    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vec), 1) =
	nxp_complex_from_gsl_complex(r2);
    GSL_RETURN(vec);
}

static NxpElement *
nxp_gsl_poly_roots_solve_quadratic(NxpElementReal * A, NxpElementReal * B,
				   NxpElementReal * C, ETC)
{
    double          a = nxp_element_r_get_value_as_double(A);
    double          b = nxp_element_r_get_value_as_double(B);
    double          c = nxp_element_r_get_value_as_double(C);
    double          r1,
                    r2;
    if (gsl_poly_solve_quadratic(a, b, c, &r1, &r2)) {
	NxpElement     *vec = nxp_element_vector_new(2, NULL);
	NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vec), 0) =
	    nxp_element_real_new(r1);
	NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(vec), 1) =
	    nxp_element_real_new(r2);
	GSL_RETURN(vec);
    } else {
	GSL_RETURN(nxp_element_vector_new(0, NULL));
    }
}

// This function evaluates the polynomial c[0] + c[1] x + c[2] x^2 + ... + 
// c[len-1] x^{len-1}
// c is a vector of reals/ints
static NxpElement *
nxp_gsl_poly_roots_poly_eval(NxpElementVector * vector_c,
			     NxpElementReal * X, ETC)
{

    double         *c = nxp_vector_to_double_buffer(vector_c, context, NULL);	/* TODO: HANDLE
									 * ERROR */
    gint            LEN;

    if (c) {
	LEN = NXP_ELEMENT_VECTOR_SIZE(vector_c);
	NxpElement     *result =
	    nxp_element_real_new(gsl_poly_eval(c, LEN, nxp_element_r_get_value_as_double(X)));

	g_free(c);
	GSL_RETURN(result);
    } else {
	GSL_RETURN(NULL);
    }
}

void
nxp_gsl_poly_roots_init(NxpEvalContext * context)
{
    NxpIdentifier  *id;
nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"poly");
id = nxp_identifier_new_with_full_path(3,"gsl","poly", "solvePolynomial");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR, nxp_gsl_poly_roots_solve_generic, NULL, nxp_gsl_poly_roots_solve_generic, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","poly", "solveCubicComplex");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_poly_roots_complex_solve_cubic, NULL, nxp_gsl_poly_roots_complex_solve_cubic, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","poly", "solveCubic");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_poly_roots_solve_cubic, NULL, nxp_gsl_poly_roots_solve_cubic, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","poly", "solveQuadraticComplex");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_poly_roots_complex_solve_quadratic, NULL, nxp_gsl_poly_roots_complex_solve_quadratic, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","poly", "solveQuadratic");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_poly_roots_solve_quadratic, NULL, nxp_gsl_poly_roots_solve_quadratic, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","poly", "evalPolynomial");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL, nxp_gsl_poly_roots_poly_eval, NULL, nxp_gsl_poly_roots_poly_eval, context, NULL); nxp_identifier_unref(id);
}
