/* -*- c -*- */
#include <gsl/gsl_sf_bessel.h>
#include "nxp_kernel.h"
#include "gsl.h"

/*
 * STATUS: FIX-MES 
 */


static NxpElement *nxp_gsl_sf_bessel_J0(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_J0(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_J1(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_J1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_Jn(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Jn(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


typedef int     (*TheFunction) (int a, int b, double x, double area[]);
typedef int     (*MaxFunction) (int a, double x, double area[]);

static NxpElement *
nxp_gsl_sf_array_generic_max(NxpElementInt * max,
			     NxpElementReal * X, MaxFunction func, ETC)
{
    nxp_int_t       nmax,
                    t;
    double         *buffer;
    int             i;
    NxpElement     *vec;

    nmax = nxp_element_r_get_value_as_int(max);
    /*
     * alloc the buffer 
     */
    buffer = g_new(double, t = nmax + 1);
    func(nmax, nxp_element_r_get_value_as_double(X), buffer);

    vec = nxp_element_vector_new(t, NULL);
    for (i = 0; i < t; i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), i) =
	    nxp_element_real_new(buffer[i]);
    }
    g_free(buffer);

    GSL_RETURN(vec);
}


static NxpElement *
nxp_gsl_sf_array_generic(NxpElementInt * min,
			 NxpElementInt * max,
			 NxpElementReal * X, TheFunction func, ETC)
{
    nxp_int_t       nmin,
                    nmax,
                    t;
    double         *buffer;
    int             i;
    NxpElement     *vec;

    nmin = nxp_element_r_get_value_as_int(min);
    nmax = nxp_element_r_get_value_as_int(max);

    /*
     * Swap them if they are swaped :-) 
     */
    if (nmin > nmax) {
	t = nmin;
	nmin = nmax;
	nmax = t;
    }

    /*
     * alloc the buffer 
     */
    buffer = g_new(double, t = nmax - nmin + 1);
    func(nmin, nmax, nxp_element_r_get_value_as_double(X), buffer);

    vec = nxp_element_vector_new(t, NULL);
    for (i = 0; i < t; i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(vec), i) =
	    nxp_element_real_new(buffer[i]);
    }
    g_free(buffer);

    GSL_RETURN(vec);
}


static NxpElement *
nxp_gsl_sf_bessel_Jn_array(NxpElementInt * min,
			   NxpElementInt * max, NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic(min, max, X, &gsl_sf_bessel_Jn_array, ETC_DELEGATE);
}


static NxpElement *nxp_gsl_sf_bessel_Y0(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Y0(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_Y1(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Y1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_Yn(NxpElementReal *arg1 ,NxpElementInt *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Yn(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_int_get_si(NXP_ELEMENT_INT(arg2), error));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_bessel_Yn_array(NxpElementInt * min,
			   NxpElementInt * max, NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic(min, max, X, &gsl_sf_bessel_Yn_array, ETC_DELEGATE);
}

static NxpElement *nxp_gsl_sf_bessel_I0(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_I0(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_I1(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_I1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_In(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_In(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_bessel_In_array(NxpElementInt * min,
			   NxpElementInt * max, NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic(min, max, X, &gsl_sf_bessel_In_array, ETC_DELEGATE);
}

static NxpElement *nxp_gsl_sf_bessel_I0_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_I0_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_I1_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_I1_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_In_scaled(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_In_scaled(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_bessel_In_scaled_array(NxpElementInt * min,
				  NxpElementInt * max,
				  NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic(min, max, X,
				    &gsl_sf_bessel_In_scaled_array, ETC_DELEGATE);
}


static NxpElement *nxp_gsl_sf_bessel_K0(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_K0(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_K1(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_K1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_Kn(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Kn(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_bessel_Kn_array(NxpElementInt * min,
			   NxpElementInt * max, NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic(min, max, X, &gsl_sf_bessel_Kn_array, ETC_DELEGATE);
}


static NxpElement *nxp_gsl_sf_bessel_K0_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_K0_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_K1_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_K1_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_Kn_scaled(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Kn_scaled(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_bessel_Kn_scaled_array(NxpElementInt * min,
				  NxpElementInt * max,
				  NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic(min, max, X,
				    &gsl_sf_bessel_Kn_scaled_array, ETC_DELEGATE);
}


static NxpElement *nxp_gsl_sf_bessel_j0(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_j0(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_j1(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_j1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_j2(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_j2(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_jl(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_jl(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_bessel_jl_array(NxpElementInt * max, NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic_max(max, X, &gsl_sf_bessel_jl_array, ETC_DELEGATE);
}

static NxpElement *
nxp_gsl_sf_bessel_jl_steed_array(NxpElementInt * max,
				 NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic_max(max, X,
					&gsl_sf_bessel_jl_steed_array, ETC_DELEGATE);
}


static NxpElement *nxp_gsl_sf_bessel_y0(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_y0(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_y1(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_y1(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_y2(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_y2(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_yl(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_yl(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_bessel_yl_array(NxpElementInt * max, NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic_max(max, X, &gsl_sf_bessel_yl_array, ETC_DELEGATE);
}


static NxpElement *nxp_gsl_sf_bessel_i0_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_i0_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_i1_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_i1_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_i2_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_i2_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_il_scaled(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_il_scaled(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_bessel_il_scaled_array(NxpElementInt * max,
				  NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic_max(max, X,
					&gsl_sf_bessel_il_scaled_array, ETC_DELEGATE);
}

static NxpElement *nxp_gsl_sf_bessel_k0_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_k0_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_k1_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_k1_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_k2_scaled(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_k2_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_kl_scaled(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_kl_scaled(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_sf_bessel_kl_scaled_array(NxpElementInt * max,
				  NxpElementReal * X, ETC)
{
    return nxp_gsl_sf_array_generic_max(max, X,
					&gsl_sf_bessel_kl_scaled_array, ETC_DELEGATE);
}

static NxpElement *nxp_gsl_sf_bessel_Jnu(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Jnu(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static int cmp_double(const void* a, const void* b) {
    double *da = (double*)a;
    double *db = (double*)b;
    return (int)(da-db);
}

static NxpElement *
nxp_gsl_sf_bessel_sequence_Jnu_e(NxpElementReal * NU,
				 NxpElementVector * V, ETC)
{
    GError         *err = NULL;
    NxpElement     *res;
    int             i;
    size_t          SIZE = NXP_ELEMENT_VECTOR_SIZE(V);
    double         *buffer = nxp_vector_to_double_buffer(V, context, &err);
    if (err) {
	g_propagate_error(error, err);
	return NULL;
    }
    qsort(buffer, SIZE, sizeof(double), cmp_double);
    gsl_sf_bessel_sequence_Jnu_e(nxp_element_r_get_value_as_double(NU), NXP_GSL_MODE, SIZE, buffer);
    res = nxp_element_vector_new(SIZE, NULL);
    for (i = 0; i < SIZE; i++) {
	_element_vector_index(NXP_ELEMENT_VECTOR(res), i) =
	    nxp_element_real_new(buffer[i]);
    }
    g_free(buffer);
    GSL_RETURN(res);

}

static NxpElement *nxp_gsl_sf_bessel_Ynu(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Ynu(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *nxp_gsl_sf_bessel_Inu(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Inu(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_Inu_scaled(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Inu_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *nxp_gsl_sf_bessel_Knu(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Knu(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_lnKnu(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_lnKnu(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_Knu_scaled(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_Knu_scaled(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *nxp_gsl_sf_bessel_zero_J0(NxpElementInt *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_zero_J0(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_zero_J1(NxpElementInt *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_zero_J1(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error));
	GSL_RETURN(nxp_element_real_new(ret));
}

static NxpElement *nxp_gsl_sf_bessel_zero_Jnu(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_sf_bessel_zero_Jnu(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


void
nxp_gsl_sf_bessel_init(NxpEvalContext * context)
{
    NxpIdentifier  *id;
nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"bessel");
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "J0");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_J0, NULL, nxp_gsl_sf_bessel_J0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "J1");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_J1, NULL, nxp_gsl_sf_bessel_J1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Jn");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_Jn, NULL, nxp_gsl_sf_bessel_Jn, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Jn");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_bessel_Jn_array, NULL, nxp_gsl_sf_bessel_Jn_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Y0");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_Y0, NULL, nxp_gsl_sf_bessel_Y0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Y1");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_Y1, NULL, nxp_gsl_sf_bessel_Y1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Yn");
 nxp_bifunc_define(id, &nxp_prototype__REAL_INT, nxp_gsl_sf_bessel_Yn, NULL, nxp_gsl_sf_bessel_Yn, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Yn");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_bessel_Yn_array, NULL, nxp_gsl_sf_bessel_Yn_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "I0");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_I0, NULL, nxp_gsl_sf_bessel_I0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "I1");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_I1, NULL, nxp_gsl_sf_bessel_I1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "In");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_In, NULL, nxp_gsl_sf_bessel_In, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "In");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_bessel_In_array, NULL, nxp_gsl_sf_bessel_In_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "I0Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_I0_scaled, NULL, nxp_gsl_sf_bessel_I0_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "I1Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_I1_scaled, NULL, nxp_gsl_sf_bessel_I1_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "InScaled");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_In_scaled, NULL, nxp_gsl_sf_bessel_In_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "InScaled");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_bessel_In_scaled_array, NULL, nxp_gsl_sf_bessel_In_scaled_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "K0");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_K0, NULL, nxp_gsl_sf_bessel_K0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "K1");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_K1, NULL, nxp_gsl_sf_bessel_K1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Kn");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_Kn, NULL, nxp_gsl_sf_bessel_Kn, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Kn");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_bessel_Kn_array, NULL, nxp_gsl_sf_bessel_Kn_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "K0Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_K0_scaled, NULL, nxp_gsl_sf_bessel_K0_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "K1Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_K1_scaled, NULL, nxp_gsl_sf_bessel_K1_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "KnScaled");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_Kn_scaled, NULL, nxp_gsl_sf_bessel_Kn_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "KnScaled");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_REAL, nxp_gsl_sf_bessel_Kn_scaled_array, NULL, nxp_gsl_sf_bessel_Kn_scaled_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "j0");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_j0, NULL, nxp_gsl_sf_bessel_j0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "j1");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_j1, NULL, nxp_gsl_sf_bessel_j1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "j2");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_j2, NULL, nxp_gsl_sf_bessel_j2, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "jl");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_jl, NULL, nxp_gsl_sf_bessel_jl, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "jl");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_jl_array, NULL, nxp_gsl_sf_bessel_jl_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "jlSteed");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_jl_steed_array, NULL, nxp_gsl_sf_bessel_jl_steed_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "y0");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_y0, NULL, nxp_gsl_sf_bessel_y0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "y1");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_y1, NULL, nxp_gsl_sf_bessel_y1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "y2");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_y2, NULL, nxp_gsl_sf_bessel_y2, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "yl");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_yl, NULL, nxp_gsl_sf_bessel_yl, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "yl");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_yl_array, NULL, nxp_gsl_sf_bessel_yl_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "i0Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_i0_scaled, NULL, nxp_gsl_sf_bessel_i0_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "i1Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_i1_scaled, NULL, nxp_gsl_sf_bessel_i1_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "i2Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_i2_scaled, NULL, nxp_gsl_sf_bessel_i2_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "ilScaled");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_il_scaled, NULL, nxp_gsl_sf_bessel_il_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "ilScaled");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_il_scaled_array, NULL, nxp_gsl_sf_bessel_il_scaled_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "k0Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_k0_scaled, NULL, nxp_gsl_sf_bessel_k0_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "k1Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_k1_scaled, NULL, nxp_gsl_sf_bessel_k1_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "k2Scaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_sf_bessel_k2_scaled, NULL, nxp_gsl_sf_bessel_k2_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "klScaled");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_kl_scaled, NULL, nxp_gsl_sf_bessel_kl_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "klScaled");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_kl_scaled_array, NULL, nxp_gsl_sf_bessel_kl_scaled_array, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Jnu");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_bessel_Jnu, NULL, nxp_gsl_sf_bessel_Jnu, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Jnu");
 nxp_bifunc_define(id, &nxp_prototype__REAL_VECTOR, nxp_gsl_sf_bessel_sequence_Jnu_e, NULL, nxp_gsl_sf_bessel_sequence_Jnu_e, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Ynu");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_bessel_Ynu, NULL, nxp_gsl_sf_bessel_Ynu, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Inu");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_bessel_Inu, NULL, nxp_gsl_sf_bessel_Inu, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "InuScaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_bessel_Inu_scaled, NULL, nxp_gsl_sf_bessel_Inu_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "Knu");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_bessel_Knu, NULL, nxp_gsl_sf_bessel_Knu, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "lnKnu");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_bessel_lnKnu, NULL, nxp_gsl_sf_bessel_lnKnu, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "KnuScaled");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_sf_bessel_Knu_scaled, NULL, nxp_gsl_sf_bessel_Knu_scaled, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "zeroJ0");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_sf_bessel_zero_J0, NULL, nxp_gsl_sf_bessel_zero_J0, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "zeroJ1");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_sf_bessel_zero_J1, NULL, nxp_gsl_sf_bessel_zero_J1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","bessel", "zeroJnu");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_sf_bessel_zero_Jnu, NULL, nxp_gsl_sf_bessel_zero_Jnu, context, NULL); nxp_identifier_unref(id);
}
