/* -*- Mode: C; c-file-style: "stroustrup"; -*- */

/* TODO: verify derive functions for ALL this functions... */

#include "nxp_kernel.h"
#include "config.h"
#include "functions.h"
#include "vector.h"
#include "matrix.h"

NxpElement*
stdlib_eval_adaptive_sample(NxpElement *expr,
                            NxpElementVariable *var,
                            NxpElementReal *nxp_xmin,
                            NxpElementReal *nxp_xmax,
                            NxpElementReal *nxp_ymin,
                            NxpElementReal *nxp_ymax,
                            NxpElementInt *nxp_precision,
                            ETC);

static void bifunc_initialize(NxpEvalContext *context, GError **error)
{
    GError *err = NULL;
    NxpElement *el;
    mpfr_t cst;

    /* init external files */
    nxp_stdlib_vector_init(context, &err);
    if (err) { g_propagate_error(error, err); return;}
    nxp_stdlib_matrix_init(context, &err);
    if (err) { g_propagate_error(error, err); return;}

    /*-( variables )--------------------------------------------------------*/
    el = nxp_element_void_new();
    nxp_built_in_variable_define_simple("void", el, context, &err);
    NXP_ELEMENT_UNREF(el);
    if (err) { g_propagate_error(error, err); return; }

    mpfr_init(cst);
    mpfr_const_pi(cst, GMP_RNDN);
    el = nxp_element_real_new_from_mpfr(cst);
    nxp_built_in_variable_define_simple("pi", el, context, &err);
    NXP_ELEMENT_UNREF(el);
    if (err) { g_propagate_error(error, err); return; }

    el = nxp_element_int_new(1);
    nxp_built_in_variable_define_simple("true", el, context, &err);
    NXP_ELEMENT_UNREF(el);
    if (err) { g_propagate_error(error, err); return; }

    el = nxp_element_int_new(0);
    nxp_built_in_variable_define_simple("false", el, context, &err);
    NXP_ELEMENT_UNREF(el);
    if (err) { g_propagate_error(error, err); return; }

    mpfr_init_set_ui(cst, 1, GMP_RNDN);
    mpfr_exp(cst, cst, GMP_RNDN);
    el = nxp_element_real_new_from_mpfr(cst);
    nxp_built_in_variable_define_simple("e", el, context, &err);
    NXP_ELEMENT_UNREF(el);
    if (err) { g_propagate_error(error, err); return; }

#define D(str, prot, eval, derive, simp) nxp_bifunc_define_global(str, \
                                         prot, eval, derive, simp, context, &err);\
                                         if (err) { g_propagate_error(error, err); return; }

    /*-( complexes )--------------------------------------------------------*/
    D("imag", &nxp_prototype__COMPLEX, stdlib_eval_im, NULL, stdlib_eval_im);
    D("real", &nxp_prototype__COMPLEX, stdlib_eval_re, NULL, stdlib_eval_re);
    D("abs", &nxp_prototype__COMPLEX, stdlib_eval_abs, NULL, NULL);
    D("arg", &nxp_prototype__COMPLEX, stdlib_eval_arg, NULL, NULL);
    D("conjugate", &nxp_prototype__COMPLEX, stdlib_eval_conjugate, NULL, stdlib_eval_conjugate);

    D("imag", &nxp_prototype__R, stdlib_eval_im_r, NULL, stdlib_eval_im_r);
    D("real", &nxp_prototype__R, stdlib_eval_re_r, NULL, stdlib_eval_re_r);
    D("abs", &nxp_prototype__R, stdlib_eval_abs_r, NULL, stdlib_eval_abs_r);
    D("arg", &nxp_prototype__R, stdlib_eval_arg_r, NULL, stdlib_eval_arg_r);

    /*-( matrices )---------------------------------------------------------*/
    D("det", &nxp_prototype__MATRIX, stdlib_eval_det, NULL, NULL);

    /*-( logic )------------------------------------------------------------*/
    D("and", &nxp_prototype__ELLIPSES, stdlib_eval_and, NULL, stdlib_simp_and);
    D("or",  &nxp_prototype__ELLIPSES, stdlib_eval_or,  NULL, stdlib_simp_or);
    D("not", &nxp_prototype__ANY,      stdlib_eval_not, NULL, stdlib_simp_not);

    /*-( erf's and erfc's )-------------------------------------------------*/
    D("erf",  &nxp_prototype__REAL, stdlib_eval_real_erf,  NULL, NULL); 
    D("erfc", &nxp_prototype__REAL, stdlib_eval_real_erfc, NULL, NULL); 

    /*-( sleep )------------------------------------------------------------*/
    D("sleep", &nxp_prototype__R, stdlib_eval_sleep, NULL, stdlib_eval_sleep);

    D("adaptivesample", &nxp_prototype__ANY_VARIABLE_REAL_REAL_REAL_REAL_INT,
      stdlib_eval_adaptive_sample, NULL, NULL);

    /*-( trignometrics )----------------------------------------------------*/
#ifdef HAVE_COMPLEX_H
#define DEF(f,op,derive) nxp_bifunc_define_global(f, &nxp_prototype__COMPLEX, \
                             stdlib_eval_complex_##op, derive, NULL, context, &err); \
                        if (err) { g_propagate_error(error, err); return; }\
                        nxp_bifunc_define_global(f, &nxp_prototype__REAL, \
                                stdlib_eval_real_##op, derive, NULL, context, &err); \
                        if (err) { g_propagate_error(error, err); return; }\
                       nxp_bifunc_define_global(f, &nxp_prototype__ANY, \
                             NULL, derive, NULL, context, &err); \
                        if (err) { g_propagate_error(error, err); return; }
#else
#define DEF(f,op,derive)  nxp_bifunc_define_global(f, &nxp_prototype__REAL, \
                                stdlib_eval_real_##op, derive, NULL, context, &err); \
                        if (err) { g_propagate_error(error, err); return; } \
                        nxp_bifunc_define_global(f, &nxp_prototype__ANY, \
                             NULL, derive, NULL, context, &err); \
                        if (err) { g_propagate_error(error, err); return; }
#endif

    DEF("sin",      sin,      stdlib_derive_sin);
    DEF("cos",	    cos,      stdlib_derive_cos);
    DEF("tan",	    tan,      stdlib_derive_tan);
    DEF("cotan",    cotg,     stdlib_derive_cotg);
    DEF("sinh",	    sinh,     stdlib_derive_sinh);
    DEF("cosh",	    cosh,     stdlib_derive_cosh);
    DEF("tanh",	    tgh,      stdlib_derive_tgh);
    DEF("cotanh",   cotgh,    stdlib_derive_cotgh);
    DEF("sec",	    sec,      stdlib_derive_sec);
    DEF("sech",     sech,     stdlib_derive_sech);
    DEF("cosec",    cosec,    stdlib_derive_cosec);
    DEF("cosech",   cosech,   stdlib_derive_cosech);
    DEF("arcsin",   arcsin,   stdlib_derive_arcsin);
    DEF("arccos",   arccos,   stdlib_derive_arccos);
    DEF("arctan",   arctg,    stdlib_derive_arctg);
    DEF("arccotan", arccotg,  stdlib_derive_arccotg);
    DEF("arcsinh",  arcsinh,  stdlib_derive_arcsinh);
    DEF("arccosh",  arccosh,  stdlib_derive_arccosh);
    DEF("arctanh",  arctgh,   stdlib_derive_arctgh);
    DEF("arccotanh",arccotgh, stdlib_derive_arccotgh);

    /* logs.c */
    DEF("sqrt",     sqrt,     stdlib_derive_sqrt);
    DEF("exp",      exp,      stdlib_derive_exp);
    DEF("log",      log,      stdlib_derive_log);
#undef DEF

    /*-( polynomials )-------------------------------------------------*/
    D("cont", &nxp_prototype__MPOLY,
      stdlib_simp_cont_mpoly, NULL, stdlib_simp_cont_mpoly);
    D("divide", &nxp_prototype__MPOLY_MPOLY,
      NULL, NULL, stdlib_simp_division_mpoly);

    /*-( integers )----------------------------------------------------*/
    D("sqrt", &nxp_prototype__INT, stdlib_eval_int_sqrt, stdlib_derive_sqrt, stdlib_simp_sqrt);
    D("isPrime", &nxp_prototype__INT,
	stdlib_eval_isprime, NULL, stdlib_eval_isprime);
    D("nextPrime", &nxp_prototype__INT,
	stdlib_eval_nextprime, NULL, stdlib_eval_nextprime);
    D("ifactor", &nxp_prototype__INT,
	stdlib_eval_ifactor, NULL, stdlib_eval_ifactor);
    D("igcd", &nxp_prototype__ELLIPSES,
	stdlib_eval_igcd, NULL, stdlib_eval_igcd);
    D("ilcm", &nxp_prototype__ELLIPSES,
	stdlib_eval_ilcm, NULL, stdlib_eval_ilcm);
#undef D
}



void nxp_module_init(NxpEvalContext *context, GError **error)
{
    GError *err = NULL;
    bifunc_initialize(context, &err);
    if (err) { g_propagate_error(error, err); return; }
}
